/***
 * Excerpted from "Core Data, 2nd Edition",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/mzcd2 for more book information.
***/
#import "AppDelegate.h"

@interface AppDelegate()

- (NSString*)applicationSupportFolder;

@end

@implementation AppDelegate

- (IBAction)addImage:(id)sender;
{
  NSManagedObject *recipe = [[recipeArrayController selectedObjects] lastObject];
  if (!recipe) return;
  NSOpenPanel *openPanel = [NSOpenPanel openPanel];
  [openPanel setCanChooseDirectories:NO];
  [openPanel setCanCreateDirectories:NO];
  [openPanel setAllowsMultipleSelection:NO];
  
  [openPanel beginSheetModalForWindow:window 
                    completionHandler:^(NSInteger result) {
    if (result == NSFileHandlingPanelCancelButton) return;
    NSURL *fileURL = [[openPanel URLs] lastObject];
    NSError *error = nil;
    //Build the path we want the file to be at
    NSURL *destURL = [NSURL fileURLWithPath:[self applicationSupportFolder]];
    NSString *guid = [[NSProcessInfo processInfo] globallyUniqueString];
    destURL = [destURL URLByAppendingPathComponent:guid];
    BOOL success = [[NSFileManager defaultManager] copyItemAtURL:fileURL 
                                                           toURL:destURL 
                                                           error:&error];
    NSAssert2(success, @"Error copying file: %@\n%@", 
              [error localizedDescription], [error userInfo]);
    [recipe setValue:[destURL path] forKey:@"imagePath"];
  }];
}

- (NSString *)applicationSupportFolder 
{
  NSArray *paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES);
  NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : NSTemporaryDirectory();
  return [basePath stringByAppendingPathComponent:@"PPRecipes"];
}

- (NSPersistentStoreCoordinator*)persistentStoreCoordinator;
{
  if (persistentStoreCoordinator) return persistentStoreCoordinator;
  
  NSManagedObjectModel *mom = nil;
  
  NSURL *modelURL = [[NSBundle mainBundle] URLForResource:@"PPRecipes" 
                                            withExtension:@"momd"];
  NSAssert(modelURL != nil, @"Unable to find DataModel in main bundle");
  mom = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
  
  NSFileManager *fileManager = [NSFileManager defaultManager];
  NSString *applicationSupportFolder = [self applicationSupportFolder];
  if ( ![fileManager fileExistsAtPath:applicationSupportFolder 
                          isDirectory:NULL] ) {
    NSError *error = nil;
    BOOL success = [fileManager createDirectoryAtPath:applicationSupportFolder 
                          withIntermediateDirectories:YES 
                                           attributes:nil 
                                                error:&error];
    NSAssert2(success, @"Error creating directory: %@\n%@", 
              [error localizedDescription], [error userInfo]);
  }
  
  NSURL *url = [NSURL fileURLWithPath:[applicationSupportFolder stringByAppendingPathComponent:@"PPRecipes.sqlite"]];


  persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] 
                                initWithManagedObjectModel:mom];
  
  NSMutableDictionary *dict = [NSMutableDictionary dictionary];
  [dict setObject:[NSNumber numberWithBool:YES] 
           forKey:NSMigratePersistentStoresAutomaticallyOption];
  
  NSError *error = nil;
  if (![persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType
                                                configuration:nil 
                                                          URL:url 
                                                      options:dict 
                                                        error:&error]) {
    NSDictionary *ui = [error userInfo];
    if (ui) {
      NSLog(@"%s(%p) %@", __PRETTY_FUNCTION__, self, [error localizedDescription]);
      for (NSError *suberror in [ui valueForKey:NSDetailedErrorsKey]) {
        NSLog(@"\t%@", [suberror localizedDescription]);
      }
    } else {
      NSLog(@"%s(%p) %@", __PRETTY_FUNCTION__, self, [error localizedDescription]);
    }
    NSAlert *alert = [[NSAlert alloc] init];
    [alert setAlertStyle:NSCriticalAlertStyle];
    [alert setMessageText:@"Unable to load the recipes database."];
    NSString *msgText = nil;
    msgText = [NSString stringWithFormat:@"The recipes database %@%@%@\n%@",
               @"is either corrupt or was created by a newer ",
               @"version of Grokking Recipes.  Please contact ",
               @"support to assist with this error.\n\nError: ",
               [error localizedDescription]];
    [alert setInformativeText:msgText];
    [alert addButtonWithTitle:@"Quit"];
    [alert runModal];
    exit(1);
  }
  return persistentStoreCoordinator;
}

- (NSManagedObjectContext*)managedObjectContext
{
  if (managedObjectContext) return managedObjectContext;
  
  NSPersistentStoreCoordinator *coord = [self persistentStoreCoordinator];
  
  if (!coord) return nil;
  
  managedObjectContext = [[NSManagedObjectContext alloc] init];
  [managedObjectContext setPersistentStoreCoordinator: coord];
  
  NSFetchRequest *request = [[NSFetchRequest alloc] init];
  [request setEntity:[NSEntityDescription entityForName:@"Type"
                                 inManagedObjectContext:managedObjectContext]];
  
  NSError *error = nil;
  NSArray *result = [managedObjectContext executeFetchRequest:request 
                                                        error:&error];
  NSAssert(error == nil, [error localizedDescription]);
  if ([result count]) return managedObjectContext;
  
  //The types table has not been populated
  NSArray *types;
  types = [[[NSBundle mainBundle] infoDictionary] valueForKey:@"RecipeTypes"];
  
  for (NSString *type in types) {
    NSManagedObject *object = [NSEntityDescription 
                               insertNewObjectForEntityForName:@"Type" 
                               inManagedObjectContext:managedObjectContext];
    [object setValue:type forKey:@"name"];
  }
  
  return managedObjectContext;
}

- (NSUndoManager *)windowWillReturnUndoManager:(NSWindow *)window
{
  return [[self managedObjectContext] undoManager];
}

- (IBAction) saveAction:(id)sender 
{
  NSError *error = nil;
  if (![[self managedObjectContext] save:&error]) {
    [[NSApplication sharedApplication] presentError:error];
  }
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(id)sender
{
  NSError *error = nil;
  
  if (!managedObjectContext) return NSTerminateNow;
  if (![managedObjectContext commitEditing]) {
    //Failed to commit editing.
    return NSTerminateCancel;
  }
  if (![managedObjectContext hasChanges]) return NSTerminateNow;
  if (![managedObjectContext save:&error]) {
    NSInteger alertReturn = NSRunAlertPanel(nil, @"Could not save changes while quitting. Quit anyway?" , @"Quit anyway", @"Cancel", nil);
    if (alertReturn == NSAlertAlternateReturn) return NSTerminateCancel;	
  }
  return NSTerminateNow;
}

@end
